@@ -1,11 +1,22 @@
 #include "All.h"
 #include "Prepare.h"
 #include "CRC.h"
+#include "GlobalFunctions.h"
+
+#if APE_BYTE_ORDER == APE_LITTLE_ENDIAN
+    #define APE_24_SHIFT_1ST 0
+    #define APE_24_SHIFT_2ND 8
+    #define APE_24_SHIFT_3RD 16
+#else
+    #define APE_24_SHIFT_1ST 16
+    #define APE_24_SHIFT_2ND 8
+    #define APE_24_SHIFT_3RD 0
+#endif
 
 namespace APE
 {
 
-int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORMATEX * pWaveFormatEx, int * pOutput, int nFrameBlocks, unsigned int * pCRC, int * pSpecialCodes, int * pPeakLevel)
+int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORMATEX * pWaveFormatEx, int * pOutput, int nFrameBlocks, unsigned int * pCRC, int * pSpecialCodes)
 {
     // error check the parameters
     if (pRawData == APE_NULL || pWaveFormatEx == APE_NULL)
@@ -21,8 +32,16 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
     int R, L;
 
     // calculate CRC
+#if APE_BYTE_ORDER == APE_BIG_ENDIAN
+    SwitchBufferBytes(const_cast<unsigned char *>(pRawData), pWaveFormatEx->wBitsPerSample / 8, nTotalBlocks * pWaveFormatEx->nChannels);
+#endif
+
     CRC = CRC_update(CRC, pRawData, nTotalBlocks * pWaveFormatEx->nChannels * (pWaveFormatEx->wBitsPerSample / 8));
 
+#if APE_BYTE_ORDER == APE_BIG_ENDIAN
+    SwitchBufferBytes(const_cast<unsigned char *>(pRawData), pWaveFormatEx->wBitsPerSample / 8, nTotalBlocks * pWaveFormatEx->nChannels);
+#endif
+
     // the prepare code
     if (pWaveFormatEx->wBitsPerSample == 32)
     {
@@ -47,10 +66,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     const int nValue = *pData;
                     pData += 1;
 
-                    // check the peak
-                    if (abs(nValue) > *pPeakLevel)
-                        *pPeakLevel = abs(nValue);
-
                     // convert to x,y
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = nValue;
                 }
@@ -66,12 +81,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 R = static_cast<int>(*pRawData++) - 128;
                 L = static_cast<int>(*pRawData++) - 128;
 
-                // check the peak
-                if (abs(L) > *pPeakLevel)
-                    *pPeakLevel = abs(L);
-                if (abs(R) > *pPeakLevel)
-                    *pPeakLevel = abs(R);
-
                 // convert to x,y
                 pOutput[nFrameBlocks + nBlockIndex] = L - R;
                 pOutput[nBlockIndex] = R + (pOutput[nFrameBlocks + nBlockIndex] / 2);
@@ -83,10 +92,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
             {
                 R = static_cast<int>(*pRawData++) - 128;
 
-                // check the peak
-                if (abs(R) > *pPeakLevel)
-                    *pPeakLevel = abs(R);
-
                 // convert to x,y
                 pOutput[nBlockIndex] = R;
             }
@@ -99,10 +104,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     R = static_cast<int>(*pRawData++) - 128;
 
-                    // check the peak
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = R;
                 }
             }
@@ -118,23 +119,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(1 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(0 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(1 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -143,23 +138,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(3 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(2 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(3 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -173,23 +162,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(1 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(0 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(1 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -198,23 +181,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(3 * nFrameBlocks) + nBlockIndex] = L;
                     pOutput[(2 * nFrameBlocks) + nBlockIndex] = R;
                 }
@@ -223,23 +200,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(5 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(4 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(5 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -249,23 +220,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     // get the value
                     R = 0;
-                    R |= (*pRawData++ << 0);
-                    R |= (*pRawData++ << 8);
-                    R |= (*pRawData++ << 16);
+                    R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    R |= (*pRawData++ << APE_24_SHIFT_3RD);
                     R = (R << 8) >> 8;
 
                     L = 0;
-                    L |= (*pRawData++ << 0);
-                    L |= (*pRawData++ << 8);
-                    L |= (*pRawData++ << 16);
+                    L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    L |= (*pRawData++ << APE_24_SHIFT_3RD);
                     L = (L << 8) >> 8;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(7 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(6 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(7 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -276,15 +241,11 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     int nValue = 0;
 
-                    nValue |= (*pRawData++ << 0);
-                    nValue |= (*pRawData++ << 8);
-                    nValue |= (*pRawData++ << 16);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_3RD);
                     nValue = (nValue << 8) >> 8;
 
-                    // check the peak
-                    if (abs(nValue) > *pPeakLevel)
-                        *pPeakLevel = abs(nValue);
-
                     // convert to x,y
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = nValue;
                 }
@@ -295,23 +256,17 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
             for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
             {
                 R = 0;
-                R |= (*pRawData++ << 0);
-                R |= (*pRawData++ << 8);
-                R |= (*pRawData++ << 16);
+                R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                R |= (*pRawData++ << APE_24_SHIFT_3RD);
                 R = (R << 8) >> 8;
 
                 L = 0;
-                L |= (*pRawData++ << 0);
-                L |= (*pRawData++ << 8);
-                L |= (*pRawData++ << 16);
+                L |= (*pRawData++ << APE_24_SHIFT_1ST);
+                L |= (*pRawData++ << APE_24_SHIFT_2ND);
+                L |= (*pRawData++ << APE_24_SHIFT_3RD);
                 L = (L << 8) >> 8;
 
-                // check the peak
-                if (abs(L) > *pPeakLevel)
-                    *pPeakLevel = abs(L);
-                if (abs(R) > *pPeakLevel)
-                    *pPeakLevel = abs(R);
-
                 // convert to x,y
                 pOutput[nFrameBlocks + nBlockIndex] = L - R;
                 pOutput[nBlockIndex] = R + (pOutput[nFrameBlocks + nBlockIndex] / 2);
@@ -322,15 +277,11 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
             for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
             {
                 R = 0;
-                R |= (*pRawData++ << 0);
-                R |= (*pRawData++ << 8);
-                R |= (*pRawData++ << 16);
+                R |= (*pRawData++ << APE_24_SHIFT_1ST);
+                R |= (*pRawData++ << APE_24_SHIFT_2ND);
+                R |= (*pRawData++ << APE_24_SHIFT_3RD);
                 R = (R << 8) >> 8;
 
-                // check the peak
-                if (abs(R) > *pPeakLevel)
-                    *pPeakLevel = abs(R);
-
                 // convert to x,y
                 pOutput[nBlockIndex] = R;
             }
@@ -343,15 +294,11 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     int nValue = 0;
 
-                    nValue |= (*pRawData++ << 0);
-                    nValue |= (*pRawData++ << 8);
-                    nValue |= (*pRawData++ << 16);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_1ST);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_2ND);
+                    nValue |= (*pRawData++ << APE_24_SHIFT_3RD);
                     nValue = (nValue << 8) >> 8;
 
-                    // check the peak
-                    if (abs(nValue) > *pPeakLevel)
-                        *pPeakLevel = abs(nValue);
-
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = nValue;
                 }
             }
@@ -373,12 +320,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(1 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(0 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(1 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -391,12 +332,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(3 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(2 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(3 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -414,12 +349,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(1 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(0 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(1 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -432,12 +361,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(3 * nFrameBlocks) + nBlockIndex] = L;
                     pOutput[(2 * nFrameBlocks) + nBlockIndex] = R;
                 }
@@ -450,12 +373,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(5 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(4 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(5 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -469,12 +386,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     L = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(L) > *pPeakLevel)
-                        *pPeakLevel = abs(L);
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(7 * nFrameBlocks) + nBlockIndex] = L - R;
                     pOutput[(6 * nFrameBlocks) + nBlockIndex] = R + (pOutput[(7 * nFrameBlocks) + nBlockIndex] / 2);
                 }
@@ -487,10 +398,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                     const int nValue = static_cast<int>(*pRawData16);
                     pRawData16++;
 
-                    // check the peak
-                    if (abs(nValue) > *pPeakLevel)
-                        *pPeakLevel = abs(nValue);
-
                     // convert to x,y
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = nValue;
                 }
@@ -519,10 +426,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
 
             if (LPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_LEFT_SILENCE; }
             if (RPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_RIGHT_SILENCE; }
-            if (ape_max(LPeak, RPeak) > *pPeakLevel)
-            {
-                *pPeakLevel = ape_max(LPeak, RPeak);
-            }
 
             // check for pseudo-stereo files
             nBlockIndex = 0;
@@ -550,8 +453,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 pOutput[nBlockIndex] = R;
             }
 
-            if (nPeak > *pPeakLevel)
-                *pPeakLevel = nPeak;
             if (nPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_MONO_SILENCE; }
         }
         else
@@ -562,10 +463,6 @@ int CPrepare::Prepare(const unsigned char * pRawData, int nBytes, const WAVEFORM
                 {
                     R = static_cast<int>(*pRawData16); pRawData16++;
 
-                    // check the peak
-                    if (abs(R) > *pPeakLevel)
-                        *pPeakLevel = abs(R);
-
                     pOutput[(nChannel * nFrameBlocks) + nBlockIndex] = R;
                 }
             }
@@ -625,13 +522,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
 
                 // surrounds
@@ -642,13 +539,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
             }
             else if (pWaveFormatEx->nChannels >= 6)
@@ -661,13 +558,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
 
                 // center and subwoofer channels
@@ -678,13 +575,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
 
                 // surrounds
@@ -695,13 +592,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
 
                 // rears
@@ -713,13 +610,13 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
                     uint32 nTempL = static_cast<uint32>(nL);
                     uint32 nTempR = static_cast<uint32>(nR);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempL >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempL >> APE_24_SHIFT_3RD) & 0xFF);
 
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTempR >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTempR >> APE_24_SHIFT_3RD) & 0xFF);
                 }
 
                 const int nStartChannel = (pWaveFormatEx->nChannels == 7) ? 7 : 8;
@@ -729,9 +626,9 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
 
                     uint32 nTemp = static_cast<uint32>(nValue);
 
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
                 }
             }
             else
@@ -742,9 +639,9 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
 
                     uint32 nTemp = static_cast<uint32>(nValue);
 
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-                    *pOutput++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+                    *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
                 }
              }
         }
@@ -910,18 +807,18 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
             else
                 nTemp = static_cast<uint32>(RV);
 
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
 
             if (LV < 0)
                 nTemp = (static_cast<uint32>((LV + 0x800000)) | 0x800000);
             else
                 nTemp = static_cast<uint32>(LV);
 
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
         }
     }
     else if (pWaveFormatEx->nChannels == 1)
@@ -947,9 +844,9 @@ void CPrepare::Unprepare(int * paryValues, const WAVEFORMATEX * pWaveFormatEx, u
             else
                 nTemp = static_cast<uint32>(RV);
 
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-            *pOutput++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+            *pOutput++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
         }
     }
 }
@@ -1015,18 +912,18 @@ int CPrepare::UnprepareOld(int * pInputX, int * pInputY, int nBlocks, const WAVE
                 else
                     nTemp = static_cast<uint32>(RV);
 
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
 
                 if (LV < 0)
                     nTemp = (static_cast<uint32>((LV + 0x800000)) | 0x800000);
                 else
                     nTemp = static_cast<uint32>(LV);
 
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
             }
         }
     }
@@ -1062,9 +959,9 @@ int CPrepare::UnprepareOld(int * pInputX, int * pInputY, int nBlocks, const WAVE
                 else
                     nTemp = static_cast<uint32>(RV);
 
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 0) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 8) & 0xFF);
-                *Buffer++ = static_cast<unsigned char>((nTemp >> 16) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_1ST) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_2ND) & 0xFF);
+                *Buffer++ = static_cast<unsigned char>((nTemp >> APE_24_SHIFT_3RD) & 0xFF);
             }
         }
         else
@@ -1080,9 +977,17 @@ int CPrepare::UnprepareOld(int * pInputX, int * pInputY, int nBlocks, const WAVE
     // calculate CRC
     uint32 CRC = 0xFFFFFFFF;
 
+#if APE_BYTE_ORDER == APE_BIG_ENDIAN
+    SwitchBufferBytes(const_cast<unsigned char *>(pRawData), pWaveFormatEx->wBitsPerSample / 8, nBlocks * pWaveFormatEx->nChannels);
+#endif
+
     CRC = CRC_update(CRC, pRawData, static_cast<int>(nBlocks * pWaveFormatEx->nChannels * (pWaveFormatEx->wBitsPerSample / 8)));
     CRC = CRC ^ 0xFFFFFFFF;
 
+#if APE_BYTE_ORDER == APE_BIG_ENDIAN
+    SwitchBufferBytes(const_cast<unsigned char *>(pRawData), pWaveFormatEx->wBitsPerSample / 8, nBlocks * pWaveFormatEx->nChannels);
+#endif
+
     *pCRC = CRC;
 
     return ERROR_SUCCESS;
